import { Component, Fragment, Suspense, Teleport } from "vue";
import { SystemProps, ResponsiveValue, StyleProps } from "@chakra-ui/styled-system";
import { IntrinsicElementAttributes } from "./dom.types";
import { AllowedComponentProps, ComponentCustomProps, VNodeProps, HTMLAttributes, VNode } from "vue";
import { DOMElements } from "./system.utils";
import { StyleResolverProps } from "./chakra";
/**
 * Export component with custom type
 *
 * @example
 * export const CBox = CBoxImpl as ComponentWithProps<{ hello?: string }>
 */
export declare type ComponentWithProps<P> = {
    new (): {
        $props: AllowedComponentProps & ComponentCustomProps & VNodeProps & {
            props?: Record<keyof P, any>;
        } & P & {
            [key: string]: unknown;
        };
    };
};
export declare type AsPolymorphicProp = {
    as?: DOMElements | string | object;
};
export declare type Tag = string | typeof Fragment | typeof Teleport | typeof Suspense | Component;
export interface ChakraProps extends SystemProps, StyleResolverProps {
    /**
     * apply layer styles defined in `theme.layerStyles`
     */
    layerStyle?: string;
    /**
     * apply typography styles defined in `theme.textStyles`
     */
    textStyle?: string;
    /**
     * Reference styles from any component or key in the theme.
     *
     * @example
     * ```html
     * <c-box apply="styles.h3">This is a div</c-box>
     * ```
     *
     * This will apply styles defined in `theme.styles.h3`
     */
    apply?: string;
    /**
     * if `true`, it'll render an ellipsis when the text exceeds the width of the viewport or maxWidth set.
     */
    isTruncated?: boolean;
    /**
     * Used to truncate text at a specific number of lines
     */
    noOfLines?: ResponsiveValue<number>;
    /**
     * Internal prop used to label Chakra factory component tags
     */
    label?: string;
    /**
     * Internal prop used to label JSX component tags
     */
    __label?: string;
}
declare type ElementType<P = any> = {
    [K in keyof IntrinsicElementAttributes]: P extends IntrinsicElementAttributes[K] ? K : never;
}[keyof IntrinsicElementAttributes] | Component<P>;
export declare type As<Props = any> = ElementType<Props>;
/**
 * Extract the props of a Vue element or component
 */
export declare type PropsOf<T extends As> = T & {
    as?: As;
};
export declare type HTMLChakraProps<T extends As> = Omit<PropsOf<T>, T extends "svg" ? "ref" | "children" | keyof StyleProps : "ref" | keyof StyleProps> & ChakraProps & {
    as?: As;
};
declare global {
    namespace h.JSX {
        interface Element extends VNode {
        }
        interface ElementClass {
            $props: {};
        }
        interface ElementAttributesProperty {
            $props: {};
        }
        interface IntrinsicAttributes extends Omit<HTMLAttributes, "color">, ChakraProps {
        }
    }
}
export {};
//# sourceMappingURL=system.types.d.ts.map